@@ -41,13 +41,13 @@
  *****************************************************************************/
 
 /* Program version and copyright message */
-#define VERSION_MESSAGE     "vlc 0.2.81 Ourumov " \
+#define VERSION_MESSAGE     "vlc 0.2.82 Ourumov " \
                           /* "(" PROGRAM_BUILD ") (" PROGRAM_OPTIONS ")\n" */ \
                             "Copyright 1996-2001 VideoLAN\n"
-#define COPYRIGHT_MESSAGE   "VideoLAN Client - version 0.2.81" \
+#define COPYRIGHT_MESSAGE   "VideoLAN Client - version 0.2.82" \
                             " Ourumov - (C)1996-2001 VideoLAN\n"
 
-#define VERSION             "0.2.81"
+#define VERSION             "0.2.82"
 
 /*****************************************************************************
  * General compilation options
@@ -112,8 +112,8 @@
  * Paths
  */
 
-#define DATA_PATH                       "/usr/local/share/videolan"
-#define PLUGIN_PATH                     "/usr/local/lib/videolan/vlc"
+#define DATA_PATH                       "/usr/share/videolan"
+#define PLUGIN_PATH                     "/usr/lib/videolan/vlc"
 
 #define MAX_PLUGIN_COUNT                32
 
@@ -412,16 +412,15 @@
 /* Better be in advance when awakening than late... */
 #define VOUT_MWAIT_TOLERANCE            ((int)(0.020*CLOCK_FREQ))
 
-/*
- * Framebuffer settings
- */
+/* The default video output window title */
+#define VOUT_TITLE                      "VideoLAN Client 0.2.82"
 
 /* Environment variable for framebuffer device, and default value */
 #define VOUT_FB_DEV_VAR                 "vlc_fb_dev"
 #define VOUT_FB_DEV_DEFAULT             "/dev/fb0"
 
-/* The default video output window title */
-#define VOUT_TITLE                      "VideoLAN Client 0.2.81"
+/* Environment variable for XVideo adaptor, and default value */
+#define VOUT_XVADAPTOR_VAR              "vlc_xv_adaptor"
 
 /*****************************************************************************
  * Video parser configuration
