@@ -4,7 +4,7 @@
  * interface, such as message output. See config.h for output configuration.
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: intf_msg.h,v 1.12 2001/04/27 19:29:11 massiot Exp $
+ * $Id: intf_msg.h,v 1.15 2001/05/31 03:12:49 sam Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -48,8 +48,13 @@ void    _intf_DbgMsgImm     ( char *psz_file, char *psz_function, int i_line,
 #else
 
 /* Non-TRACE mode */
-#define intf_DbgMsg( format, args... )
-#define intf_DbgMsgImm( format, args...)
+#if defined( _MSC_VER )
+#   define intf_DbgMsg
+#   define intf_DbgMsgImm
+#else
+#   define intf_DbgMsg( format, args... )
+#   define intf_DbgMsgImm( format, args...)
+#endif
 
 #endif
 
@@ -81,7 +86,6 @@ void         intf_MsgDestroy     ( void );
 void         intf_Msg            ( char *psz_format, ... );
 void         intf_ErrMsg         ( char *psz_format, ... );
 void         intf_WarnMsg        ( int i_level, char *psz_format, ... );
-void         intf_IntfMsg        ( char *psz_format, ... );
 
 void         intf_MsgImm         ( char *psz_format, ... );
 void         intf_ErrMsgImm      ( char *psz_format, ... );
