@@ -4,7 +4,7 @@
  * includes all common video types and constants.
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: video.h,v 1.29 2001/03/21 13:42:33 sam Exp $
+ * $Id: video.h,v 1.31 2001/07/18 14:21:00 massiot Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -71,11 +71,6 @@ typedef struct picture_s
     int             i_display_height;               /* useful picture height */
     int             i_aspect_ratio;                          /* aspect ratio */
 
-    /* Macroblock counter - the decoder use it to verify if it has
-     * decoded all the macroblocks of the picture */
-    int             i_deccount;
-    vlc_mutex_t     lock_deccount;
-
     /* Picture data - data can always be freely modified. p_data itself
      * (the pointer) should NEVER be modified. In YUV format, the p_y, p_u and
      * p_v data pointers refers to different areas of p_data, and should not
@@ -123,9 +118,10 @@ typedef struct subpicture_s
     int             i_size;                                     /* data size */
     struct subpicture_s *   p_next;         /* next subtitle to be displayed */
 
-    /* Other properties */
-    mtime_t         begin_date;                 /* beginning of display date */
-    mtime_t         end_date;                         /* end of display date */
+    /* Date properties */
+    mtime_t         i_start;                    /* beginning of display date */
+    mtime_t         i_stop;                           /* end of display date */
+    boolean_t       b_ephemer;             /* does the subtitle have a TTL ? */
 
     /* Display properties - these properties are only indicative and may be
      * changed by the video output thread, or simply ignored depending of the
