@@ -2,7 +2,7 @@
  * idctclassic.c : Classic IDCT module
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: idctclassic.c,v 1.8 2001/04/15 04:19:57 sam Exp $
+ * $Id: idctclassic.c,v 1.13 2001/07/17 09:48:07 massiot Exp $
  *
  * Authors: Gal Hendryckx <jimmy@via.ecp.fr>
  *
@@ -39,12 +39,15 @@
 #include "video.h"
 #include "video_output.h"
 
-#include "video_decoder.h"
+#include "vdec_ext-plugins.h"
 
 #include "modules.h"
 #include "modules_inner.h"
 
-#include "idct.h"
+#include "vdec_block.h"
+#include "vdec_idct.h"
+
+#include "modules_export.h"
 
 /*****************************************************************************
  * Local and extern prototypes.
@@ -60,64 +63,20 @@ static void vdec_NormScan   ( u8 ppi_scan[2][64] );
 MODULE_CONFIG_START
 ADD_WINDOW( "Configuration for classic IDCT module" )
     ADD_COMMENT( "Ha, ha -- nothing to configure yet" )
-MODULE_CONFIG_END
-
-/*****************************************************************************
- * InitModule: get the module structure and configuration.
- *****************************************************************************
- * We have to fill psz_name, psz_longname and psz_version. These variables
- * will be strdup()ed later by the main application because the module can
- * be unloaded later to save memory, and we want to be able to access this
- * data even after the module has been unloaded.
- *****************************************************************************/
-MODULE_INIT
-{
-    p_module->psz_name = MODULE_STRING;
-    p_module->psz_longname = "classic IDCT module";
-    p_module->psz_version = VERSION;
+MODULE_CONFIG_STOP
 
+MODULE_INIT_START
     p_module->i_capabilities = MODULE_CAPABILITY_NULL
                                 | MODULE_CAPABILITY_IDCT;
+    p_module->psz_longname = "classic IDCT module";
+MODULE_INIT_STOP
 
-    return( 0 );
-}
-
-/*****************************************************************************
- * ActivateModule: set the module to an usable state.
- *****************************************************************************
- * This function fills the capability functions and the configuration
- * structure. Once ActivateModule() has been called, the i_usage can
- * be set to 0 and calls to NeedModule() be made to increment it. To unload
- * the module, one has to wait until i_usage == 0 and call DeactivateModule().
- *****************************************************************************/
-MODULE_ACTIVATE
-{
-    p_module->p_functions = malloc( sizeof( module_functions_t ) );
-    if( p_module->p_functions == NULL )
-    {
-        return( -1 );
-    }
-
+MODULE_ACTIVATE_START
     idct_getfunctions( &p_module->p_functions->idct );
+MODULE_ACTIVATE_STOP
 
-    p_module->p_config = p_config;
-
-    return( 0 );
-}
-
-/*****************************************************************************
- * DeactivateModule: make sure the module can be unloaded.
- *****************************************************************************
- * This function must only be called when i_usage == 0. If it successfully
- * returns, i_usage can be set to -1 and the module unloaded. Be careful to
- * lock usage_lock during the whole process.
- *****************************************************************************/
-MODULE_DEACTIVATE
-{
-    free( p_module->p_functions );
-
-    return( 0 );
-}
+MODULE_DEACTIVATE_START
+MODULE_DEACTIVATE_STOP
 
 /* Following functions are local */
 
@@ -128,10 +87,15 @@ MODULE_DEACTIVATE
 static void idct_getfunctions( function_list_t * p_function_list )
 {
     p_function_list->pf_probe = idct_Probe;
-    p_function_list->functions.idct.pf_init = _M( vdec_InitIDCT );
-    p_function_list->functions.idct.pf_sparse_idct = _M( vdec_SparseIDCT );
-    p_function_list->functions.idct.pf_idct = _M( vdec_IDCT );
-    p_function_list->functions.idct.pf_norm_scan = vdec_NormScan;
+#define F p_function_list->functions.idct
+    F.pf_idct_init = _M( vdec_InitIDCT );
+    F.pf_sparse_idct = _M( vdec_SparseIDCT );
+    F.pf_idct = _M( vdec_IDCT );
+    F.pf_norm_scan = vdec_NormScan;
+    F.pf_decode_init = _M( vdec_InitDecode );
+    F.pf_decode_mb_c = _M( vdec_DecodeMacroblockC );
+    F.pf_decode_mb_bw = _M( vdec_DecodeMacroblockBW );
+#undef F
 }
 
 /*****************************************************************************
@@ -139,7 +103,8 @@ static void idct_getfunctions( function_list_t * p_function_list )
  *****************************************************************************/
 static int idct_Probe( probedata_t *p_data )
 {
-    if( TestMethod( IDCT_METHOD_VAR, "idctclassic" ) )
+    if( TestMethod( IDCT_METHOD_VAR, "idctclassic" )
+         || TestMethod( IDCT_METHOD_VAR, "classic" ) )
     {
         return( 999 );
     }
@@ -158,8 +123,7 @@ static void vdec_NormScan( u8 ppi_scan[2][64] )
 /*****************************************************************************
  * vdec_IDCT : IDCT function for normal matrices
  *****************************************************************************/
-void _M( vdec_IDCT )( vdec_thread_t * p_vdec, dctelem_t * p_block,
-                int i_idontcare )
+void _M( vdec_IDCT )( void * p_idct_data, dctelem_t * p_block, int i_idontcare )
 {
     /* dct classique: pour tester la meilleure entre la classique et la */
     /* no classique */
