@@ -2,7 +2,7 @@
  * input_ps.c: PS demux and packet management
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: input_ps.c,v 1.30 2001/07/17 09:48:07 massiot Exp $
+ * $Id: input_ps.c,v 1.34 2001/08/14 04:52:39 sam Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Cyril Deguet <asmax@via.ecp.fr>
@@ -31,6 +31,7 @@
 #include "defs.h"
 
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <errno.h>
 
@@ -71,6 +72,19 @@
 #include "modules.h"
 #include "modules_export.h"
 
+/*****************************************************************************
+ * fseeko: fseeko replacement for BSDI.
+ *****************************************************************************/
+#ifdef __bsdi__
+int    __sfseek __P(( FILE *, fpos_t, int ));
+fpos_t __sftell __P(( FILE * ));
+
+static __inline__ off_t fseeko( FILE *p_file, off_t i_offset, int i_pos )
+{
+    return __sfseek( p_file, i_offset, i_pos );
+}
+#endif
+
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
@@ -387,7 +401,22 @@ static void PSInit( input_thread_t * p_input )
  *****************************************************************************/
 static void PSEnd( input_thread_t * p_input )
 {
-    vlc_mutex_destroy( &((packet_cache_t *)p_input->p_plugin_data)->lock );
+#define p_packet_cache ((packet_cache_t *)p_input->p_method_data)
+
+    vlc_mutex_destroy( &p_packet_cache->lock );
+
+    if( p_packet_cache->data.p_stack )
+        free( p_packet_cache->data.p_stack );
+    if( p_packet_cache->pes.p_stack )
+        free( p_packet_cache->pes.p_stack );
+    if( p_packet_cache->smallbuffer.p_stack )
+        free( p_packet_cache->smallbuffer.p_stack );
+    if( p_packet_cache->largebuffer.p_stack )
+        free( p_packet_cache->largebuffer.p_stack );
+
+#undef p_packet_cache
+
+    free( p_input->p_method_data );
     free( p_input->p_plugin_data );
 }
 
@@ -441,7 +470,7 @@ static int PSRead( input_thread_t * p_input,
     for( i_packet = 0; i_packet < INPUT_READ_ONCE; i_packet++ )
     {
         /* Read what we believe to be a packet header. */
-        if( (i_error = SafeRead( p_input, p_header, 6 )) )
+        if( (i_error = SafeRead( p_input, p_header, 4 )) )
         {
             return( i_error );
         }
@@ -475,36 +504,47 @@ static int PSRead( input_thread_t * p_input,
             }
             /* Packet found. */
             *(u32 *)p_header = U32_AT(&i_startcode);
+        }
+
+        /* 0x1B9 == SYSTEM_END_CODE, it is only 4 bytes long. */
+        if( U32_AT(p_header) != 0x1B9 )
+        {
+            /* The packet is at least 6 bytes long. */
             if( (i_error = SafeRead( p_input, p_header + 4, 2 )) )
             {
                 return( i_error );
             }
-        }
 
-        if( U32_AT(p_header) != 0x1BA )
-        {
-            /* That's the case for all packets, except pack header. */
-            i_packet_size = U16_AT(&p_header[4]);
-        }
-        else
-        {
-            /* Pack header. */
-            if( (p_header[4] & 0xC0) == 0x40 )
+            if( U32_AT(p_header) != 0x1BA )
             {
-                /* MPEG-2 */
-                i_packet_size = 8;
-            }
-            else if( (p_header[4] & 0xF0) == 0x20 )
-            {
-                /* MPEG-1 */
-                i_packet_size = 6;
+                /* That's the case for all packets, except pack header. */
+                i_packet_size = U16_AT(&p_header[4]);
             }
             else
             {
-                intf_ErrMsg( "Unable to determine stream type" );
-                return( -1 );
+                /* Pack header. */
+                if( (p_header[4] & 0xC0) == 0x40 )
+                {
+                    /* MPEG-2 */
+                    i_packet_size = 8;
+                }
+                else if( (p_header[4] & 0xF0) == 0x20 )
+                {
+                    /* MPEG-1 */
+                    i_packet_size = 6;
+                }
+                else
+                {
+                    intf_ErrMsg( "Unable to determine stream type" );
+                    return( -1 );
+                }
             }
         }
+        else
+        {
+            /* System End Code */
+            i_packet_size = -2;
+        }
 
         /* Fetch a packet of the appropriate size. */
         p_data = NewPacket( p_input->p_method_data, i_packet_size + 6 );
@@ -514,30 +554,38 @@ static int PSRead( input_thread_t * p_input,
             return( -1 );
         }
 
-        /* Copy the header we already read. */
-        memcpy( p_data->p_buffer, p_header, 6 );
-
-        /* Read the remaining of the packet. */
-        if( i_packet_size && (i_error =
-                SafeRead( p_input, p_data->p_buffer + 6, i_packet_size )) )
+        if( U32_AT(p_header) != 0x1B9 )
         {
-            return( i_error );
-        }
+            /* Copy the header we already read. */
+            memcpy( p_data->p_buffer, p_header, 6 );
 
-        /* In MPEG-2 pack headers we still have to read stuffing bytes. */
-        if( U32_AT(p_header) == 0x1BA )
-        {
-            if( i_packet_size == 8 && (p_data->p_buffer[13] & 0x7) != 0 )
+            /* Read the remaining of the packet. */
+            if( i_packet_size && (i_error =
+                    SafeRead( p_input, p_data->p_buffer + 6, i_packet_size )) )
             {
-                /* MPEG-2 stuffing bytes */
-                byte_t      p_garbage[8];
-                if( (i_error = SafeRead( p_input, p_garbage,
-                                         p_data->p_buffer[13] & 0x7)) )
+                return( i_error );
+            }
+
+            /* In MPEG-2 pack headers we still have to read stuffing bytes. */
+            if( U32_AT(p_header) == 0x1BA )
+            {
+                if( i_packet_size == 8 && (p_data->p_buffer[13] & 0x7) != 0 )
                 {
-                    return( i_error );
+                    /* MPEG-2 stuffing bytes */
+                    byte_t      p_garbage[8];
+                    if( (i_error = SafeRead( p_input, p_garbage,
+                                             p_data->p_buffer[13] & 0x7)) )
+                    {
+                        return( i_error );
+                    }
                 }
             }
         }
+        else
+        {
+            /* Copy the small header. */
+            memcpy( p_data->p_buffer, p_header, 4 );
+        }
 
         /* Give the packet to the other input stages. */
         pp_packets[i_packet] = p_data;
@@ -556,10 +604,10 @@ static void PSSeek( input_thread_t * p_input, off_t i_position )
     p_method = (thread_ps_data_t *)p_input->p_plugin_data;
 
     /* A little bourrin but should work for a while --Meuuh */
-#ifndef WIN32
-    fseeko( p_method->stream, i_position, SEEK_SET );
-#else
+#if defined( WIN32 ) || defined( SYS_GNU0_2 )
     fseek( p_method->stream, (long)i_position, SEEK_SET );
+#else
+    fseeko( p_method->stream, i_position, SEEK_SET );
 #endif
 
     p_input->stream.p_selected_area->i_tell = i_position;
