@@ -4,7 +4,7 @@
  * and spawn threads.
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: main.c,v 1.85 2001/04/12 01:52:45 sam Exp $
+ * $Id: main.c,v 1.112 2001/08/22 14:23:57 sam Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -31,6 +31,7 @@
 
 #include <signal.h>                               /* SIGHUP, SIGINT, SIGKILL */
 #include <stdio.h>                                              /* sprintf() */
+#include <setjmp.h>                                       /* longjmp, setjmp */
 
 #ifdef HAVE_GETOPT_LONG
 #   ifdef HAVE_GETOPT_H
@@ -40,17 +41,28 @@
 #   include "GNUgetopt/getopt.h"
 #endif
 
-#ifdef SYS_DARWIN1_3
+#ifdef SYS_DARWIN
 #   include <mach/mach.h>                               /* Altivec detection */
 #   include <mach/mach_error.h>       /* some day the header files||compiler *
                                                        will define it for us */
 #   include <mach/bootstrap.h>
 #endif
 
-#include <unistd.h>
+#ifndef WIN32
+#   include <netinet/in.h>                            /* BSD: struct in_addr */
+#endif
+
+#ifdef HAVE_UNISTD_H
+#   include <unistd.h>
+#elif defined( _MSC_VER ) && defined( _WIN32 )
+#   include <io.h>
+#endif
+
 #include <errno.h>                                                 /* ENOMEM */
 #include <stdlib.h>                                  /* getenv(), strtol(),  */
 #include <string.h>                                            /* strerror() */
+#include <fcntl.h>                                       /* open(), O_WRONLY */
+#include <sys/stat.h>                                             /* S_IREAD */
 
 #include "config.h"
 #include "common.h"
@@ -76,7 +88,7 @@
 #   include "beos_specific.h"
 #endif
 
-#ifdef SYS_DARWIN1_3
+#ifdef SYS_DARWIN
 #   include "darwin_specific.h"
 #endif
 
@@ -94,6 +106,7 @@
 #define OPT_NOAUDIO             150
 #define OPT_STEREO              151
 #define OPT_MONO                152
+#define OPT_SPDIF               153
 
 #define OPT_NOVIDEO             160
 #define OPT_DISPLAY             161
@@ -102,6 +115,8 @@
 #define OPT_COLOR               164
 #define OPT_FULLSCREEN          165
 #define OPT_OVERLAY             166
+#define OPT_XVADAPTOR           167
+#define OPT_SMP                 168
 
 #define OPT_CHANNELS            170
 #define OPT_SERVER              171
@@ -112,16 +127,29 @@
 #define OPT_MOTION              181
 #define OPT_IDCT                182
 #define OPT_YUV                 183
+#define OPT_DOWNMIX             184
+#define OPT_IMDCT               185
 
 #define OPT_SYNCHRO             190
 #define OPT_WARNING             191
 #define OPT_VERSION             192
+#define OPT_STDOUT              193
 
 /* Usage fashion */
 #define USAGE                     0
 #define SHORT_HELP                1
 #define LONG_HELP                 2
 
+/* Needed for x86 CPU capabilities detection */
+#define cpuid( a )                 \
+    asm volatile ( "cpuid"         \
+                 : "=a" ( i_eax ), \
+                   "=b" ( i_ebx ), \
+                   "=c" ( i_ecx ), \
+                   "=d" ( i_edx )  \
+                 : "a"  ( a )      \
+                 : "cc" );
+
 /* Long options */
 static const struct option longopts[] =
 {
@@ -135,12 +163,16 @@ static const struct option longopts[] =
     /* Interface options */
     {   "intf",             1,          0,      'I' },
     {   "warning",          1,          0,      OPT_WARNING },
+    {   "stdout",           1,          0,      OPT_STDOUT },
 
     /* Audio options */
     {   "noaudio",          0,          0,      OPT_NOAUDIO },
     {   "aout",             1,          0,      'A' },
     {   "stereo",           0,          0,      OPT_STEREO },
     {   "mono",             0,          0,      OPT_MONO },
+    {   "spdif",            0,          0,      OPT_SPDIF },
+    {   "downmix",          1,          0,      OPT_DOWNMIX },
+    {   "imdct",            1,          0,      OPT_IMDCT },
 
     /* Video options */
     {   "novideo",          0,          0,      OPT_NOVIDEO },
@@ -155,20 +187,23 @@ static const struct option longopts[] =
     {   "yuv",              1,          0,      OPT_YUV },
     {   "fullscreen",       0,          0,      OPT_FULLSCREEN },
     {   "overlay",          0,          0,      OPT_OVERLAY },
+    {   "xvadaptor",        1,          0,      OPT_XVADAPTOR },
+    {   "smp",              1,          0,      OPT_SMP },
 
     /* DVD options */
     {   "dvdtitle",         1,          0,      't' },
     {   "dvdchapter",       1,          0,      'T' },
+    {   "dvdangle",         1,          0,      'u' },
     {   "dvdaudio",         1,          0,      'a' },
     {   "dvdchannel",       1,          0,      'c' },
     {   "dvdsubtitle",      1,          0,      's' },
     
     /* Input options */
     {   "input",            1,          0,      OPT_INPUT },
-    {   "channels",         0,          0,      OPT_CHANNELS },
     {   "server",           1,          0,      OPT_SERVER },
     {   "port",             1,          0,      OPT_PORT },
-    {   "broadcast",        0,          0,      OPT_BROADCAST },
+    {   "broadcast",        1,          0,      OPT_BROADCAST },
+    {   "channels",         0,          0,      OPT_CHANNELS },
 
     /* Synchro options */
     {   "synchro",          1,          0,      OPT_SYNCHRO },
@@ -176,12 +211,16 @@ static const struct option longopts[] =
 };
 
 /* Short options */
-static const char *psz_shortopts = "hHvgt:T:a:s:c:I:A:V:";
+static const char *psz_shortopts = "hHvgt:T:u:a:s:c:I:A:V:";
 
 /*****************************************************************************
- * Global variable program_data - this is the one and only, see main.h
+ * Global variable program_data - these are the only ones, see main.h and
+ * modules.h
  *****************************************************************************/
-main_t *p_main;
+main_t        *p_main;
+module_bank_t *p_module_bank;
+aout_bank_t   *p_aout_bank;
+vout_bank_t   *p_vout_bank;
 
 /*****************************************************************************
  * Local prototypes
@@ -195,9 +234,15 @@ static void Version                 ( void );
 static void InitSignalHandler       ( void );
 static void SimpleSignalHandler     ( int i_signal );
 static void FatalSignalHandler      ( int i_signal );
-
+static void InstructionSignalHandler( int i_signal );
 static int  CPUCapabilities         ( void );
 
+static int  RedirectSTDOUT          ( void );
+static void ShowConsole             ( void );
+
+static jmp_buf env;
+static int  i_illegal;
+
 /*****************************************************************************
  * main: parse command line, start interface and spawn threads
  *****************************************************************************
@@ -210,22 +255,26 @@ static int  CPUCapabilities         ( void );
  *****************************************************************************/
 int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
 {
-    main_t  main_data;                      /* root of all data - see main.h */
+    main_t        main_data;                /* root of all data - see main.h */
+    module_bank_t module_bank;
+    aout_bank_t   aout_bank;
+    vout_bank_t   vout_bank;
 
-    p_main = &main_data;                       /* set up the global variable */
+    p_main        = &main_data;               /* set up the global variables */
+    p_module_bank = &module_bank;
+    p_aout_bank   = &aout_bank;
+    p_vout_bank   = &vout_bank;
 
     /*
-     * System specific initialization code
+     * Initialize threads system
      */
-#if defined( SYS_BEOS ) || defined( SYS_DARWIN1_3 )
-    system_Create( &i_argc, ppsz_argv, ppsz_env );
-#endif
+    vlc_threads_init( );
 
-    p_main->i_cpu_capabilities = CPUCapabilities();
-    
     /*
      * Test if our code is likely to run on this CPU 
      */
+    p_main->i_cpu_capabilities = CPUCapabilities();
+    
 #if defined( __pentium__ ) || defined( __pentiumpro__ )
     if( ! TestCPU( CPU_CAPABILITY_586 ) )
     {
@@ -235,13 +284,13 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
     }
 #endif
 
-#ifdef HAVE_MMX
-    if( ! TestCPU( CPU_CAPABILITY_MMX ) )
-    {
-        fprintf( stderr, "error: this program needs MMX extensions,\n"
-                         "please try a version without MMX support\n" );
-        return( 1 );
-    }
+    /*
+     * System specific initialization code
+     */
+#if defined( SYS_BEOS ) || defined( SYS_DARWIN )
+    system_Init( &i_argc, ppsz_argv, ppsz_env );
+#elif defined( WIN32 )
+    _fmode = _O_BINARY;   /* sets the default file-translation mode on Win32 */
 #endif
 
     /*
@@ -260,16 +309,22 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
     /*
      * Read configuration
      */
-    if( GetConfiguration( &i_argc, ppsz_argv, ppsz_env ) )  /* parse cmd line */
+    if( GetConfiguration( &i_argc, ppsz_argv, ppsz_env ) ) /* parse cmd line */
     {
         intf_MsgDestroy();
         return( errno );
     }
 
+    /*
+     * Redirect the standard output if required by the user, and on Win32 we
+     * also open a console to display the debug messages.
+     */
+    RedirectSTDOUT();
+
     /*
      * Initialize playlist and get commandline files
      */
-    p_main->p_playlist = intf_PlaylistCreate( );
+    p_main->p_playlist = intf_PlaylistCreate();
     if( !p_main->p_playlist )
     {
         intf_ErrMsg( "playlist error: playlist initialization failed" );
@@ -284,115 +339,66 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
     GetFilenames( i_argc, ppsz_argv );
 
     /*
-     * Initialize module bank
+     * Initialize module, aout and vout banks
      */
-    p_main->p_bank = module_CreateBank( );
-    if( !p_main->p_bank )
-    {
-        intf_ErrMsg( "module error: module bank initialization failed" );
-        intf_PlaylistDestroy( p_main->p_playlist );
-        intf_MsgDestroy();
-        return( errno );
-    }
-    module_InitBank( p_main->p_bank );
+    module_InitBank();
+    aout_InitBank();
+    vout_InitBank();
 
     /*
      * Initialize shared resources and libraries
      */
-    if( p_main->b_channels && network_ChannelCreate() )
+    if( main_GetIntVariable( INPUT_NETWORK_CHANNEL_VAR,
+                             INPUT_NETWORK_CHANNEL_DEFAULT ) &&
+        network_ChannelCreate() )
     {
         /* On error during Channels initialization, switch off channels */
         intf_Msg( "Channels initialization failed : "
                   "Channel management is deactivated" );
-        p_main->b_channels = 0;
+        main_PutIntVariable( INPUT_NETWORK_CHANNEL_VAR, 0 );
     }
 
     /*
-     * Run interface
+     * Try to run the interface
      */
     p_main->p_intf = intf_Create();
-    if( !p_main->p_intf )
+    if( p_main->p_intf == NULL )
     {
         intf_ErrMsg( "intf error: interface initialization failed" );
-        module_DestroyBank( p_main->p_bank );
-        intf_PlaylistDestroy( p_main->p_playlist );
-        intf_MsgDestroy();
-        return( errno );
     }
-
-    /*
-     * Set signal handling policy for all threads
-     */
-    InitSignalHandler();
-
-    /*
-     * Open audio device and start aout thread
-     */
-    if( p_main->b_audio )
+    else
     {
-        p_main->p_aout = aout_CreateThread( NULL );
-        if( p_main->p_aout == NULL )
+        /*
+         * Set signal handling policy for all threads
+         */
+        InitSignalHandler();
+
+        /*
+         * This is the main loop
+         */
+        p_main->p_intf->pf_run( p_main->p_intf );
+
+        /*
+         * Finished, destroy the interface
+         */
+        intf_Destroy( p_main->p_intf );
+
+        /*
+         * Go back into channel 0 which is the network
+         */
+        if( main_GetIntVariable( INPUT_NETWORK_CHANNEL_VAR,
+                                 INPUT_NETWORK_CHANNEL_DEFAULT ) )
         {
-            /* On error during audio initialization, switch off audio */
-            intf_ErrMsg( "aout error: audio initialization failed,"
-                         " audio is deactivated" );
-            p_main->b_audio = 0;
+            network_ChannelJoin( COMMON_CHANNEL );
         }
     }
 
     /*
-     * Open video device and start vout thread
+     * Free module, aout and vout banks
      */
-    if( p_main->b_video )
-    {
-        p_main->p_vout = vout_CreateThread( NULL );
-        if( p_main->p_vout == NULL )
-        {
-            /* On error during video initialization, switch off video */
-            intf_ErrMsg( "vout error: video initialization failed,"
-                         " video is deactivated" );
-            p_main->b_video = 0;
-        }
-    }
-
-    /* Flush messages before entering the main loop */
-    intf_FlushMsg();
-
-    /*
-     * This is the main loop
-     */
-    p_main->p_intf->pf_run( p_main->p_intf );
-
-    intf_Destroy( p_main->p_intf );
-
-    /*
-     * Close video device
-     */
-    if( p_main->b_video )
-    {
-        vout_DestroyThread( p_main->p_vout, NULL );
-    }
-
-    /*
-     * Close audio device
-     */
-    if( p_main->b_audio )
-    {
-        aout_DestroyThread( p_main->p_aout, NULL );
-    }
-
-    /*
-     * Go back into channel 0 which is the network
-     */
-    if( p_main->b_channels )
-    {
-        network_ChannelJoin( COMMON_CHANNEL );
-    }
-
-    /*
-     * Free module bank
-     */
-    module_DestroyBank( p_main->p_bank );
+    vout_EndBank();
+    aout_EndBank();
+    module_EndBank();
 
     /*
      * Free playlist
@@ -402,17 +408,23 @@ int main( int i_argc, char *ppsz_argv[], char *ppsz_env[] )
     /*
      * System specific cleaning code
      */
-#if defined( SYS_BEOS ) || defined( SYS_DARWIN1_3 )
-    system_Destroy();
+#if defined( SYS_BEOS ) || defined( SYS_DARWIN )
+    system_End();
 #endif
 
+
     /*
      * Terminate messages interface and program
      */
     intf_Msg( "intf: program terminated" );
     intf_MsgDestroy();
 
-    return( 0 );
+    /*
+     * Stop threads system
+     */
+    vlc_threads_end( );
+
+    return 0;
 }
 
 /*****************************************************************************
@@ -520,9 +532,10 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
 
     p_main->b_audio     = 1;
     p_main->b_video     = 1;
-    p_main->b_channels  = 0;
 
-    p_main->i_warning_level = 4;
+    p_main->i_warning_level = 0;
+
+    p_main->p_channel = NULL;
 
     /* Get the executable name (similar to the basename command) */
     p_main->psz_arg0 = p_tmp = ppsz_argv[ 0 ];
@@ -538,12 +551,12 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
         }
     }
 
-#ifdef SYS_DARWIN1_3
+#ifdef SYS_DARWIN
     /* When vlc.app is run by double clicking in Mac OS X, the 2nd arg
      * is the PSN - process serial number (a unique PID-ish thingie)
      * still ok for real Darwin & when run from command line */
-    if ( strncmp( ppsz_argv[ 1 ] , "-psn" , 4) == 0 )
-                                  /* for example -psn_0_9306113 */
+    if ( (*pi_argc > 1) && (strncmp( ppsz_argv[ 1 ] , "-psn" , 4 ) == 0) )
+                                        /* for example -psn_0_9306113 */
     {
         /* GDMF!... I can't do this or else the MacOSX window server will
          * not pick up the PSN and not register the app and we crash...
@@ -567,19 +580,52 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
         {
         /* General/common options */
         case 'h':                                              /* -h, --help */
+            ShowConsole();
+            RedirectSTDOUT();
             Usage( SHORT_HELP );
+#ifdef WIN32        /* Pause the console because it's destroyed when we exit */
+            if( strcmp( "", main_GetPszVariable( INTF_STDOUT_VAR,
+                                                 INTF_STDOUT_DEFAULT ) ) == 0 )
+            {
+                /* No stdout redirection has been asked for */
+                intf_MsgImm( "\nPress the RETURN key to continue..." );
+                getchar();
+            }
+#endif
             return( -1 );
             break;
         case 'H':                                          /* -H, --longhelp */
+            ShowConsole();
+            RedirectSTDOUT();
             Usage( LONG_HELP );
+#ifdef WIN32        /* Pause the console because it's destroyed when we exit */
+            if( strcmp( "", main_GetPszVariable( INTF_STDOUT_VAR,
+                                                 INTF_STDOUT_DEFAULT ) ) == 0 )
+            {
+                /* No stdout redirection has been asked for */
+                intf_MsgImm( "\nPress the RETURN key to continue..." );
+                getchar();
+            }
+#endif
             return( -1 );
             break;
         case OPT_VERSION:                                       /* --version */
+            ShowConsole();
+            RedirectSTDOUT();
             Version();
+#ifdef WIN32        /* Pause the console because it's destroyed when we exit */
+            if( strcmp( "", main_GetPszVariable( INTF_STDOUT_VAR,
+                                                 INTF_STDOUT_DEFAULT ) ) == 0 )
+            {
+                /* No stdout redirection has been asked for */
+                intf_MsgImm( "\nPress the RETURN key to continue..." );
+                getchar();
+            }
+#endif
             return( -1 );
             break;
         case 'v':                                           /* -v, --verbose */
-            p_main->i_warning_level--;
+            p_main->i_warning_level++;
             break;
 
         /* Interface warning messages level */
@@ -591,6 +637,10 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
             p_main->i_warning_level = atoi(optarg);
             break;
 
+        case OPT_STDOUT:                                         /* --stdout */
+            main_PutPszVariable( INTF_STDOUT_VAR, optarg );
+            break;
+
         /* Audio options */
         case OPT_NOAUDIO:                                       /* --noaudio */
             p_main->b_audio = 0;
@@ -604,6 +654,15 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
         case OPT_MONO:                                             /* --mono */
             main_PutIntVariable( AOUT_STEREO_VAR, 0 );
             break;
+        case OPT_SPDIF:                                           /* --spdif */
+            main_PutIntVariable( AOUT_SPDIF_VAR, 1 );
+            break;
+        case OPT_DOWNMIX:                                       /* --downmix */
+            main_PutPszVariable( DOWNMIX_METHOD_VAR, optarg );
+            break;
+        case OPT_IMDCT:                                           /* --imdct */
+            main_PutPszVariable( IMDCT_METHOD_VAR, optarg );
+            break;
 
         /* Video options */
         case OPT_NOVIDEO:                                       /* --novideo */
@@ -633,6 +692,9 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
         case OPT_OVERLAY:                                       /* --overlay */
             main_PutIntVariable( VOUT_OVERLAY_VAR, 1 );
             break;
+        case OPT_XVADAPTOR:                                   /* --xvadaptor */
+            main_PutIntVariable( VOUT_XVADAPTOR_VAR, atoi(optarg) );
+            break;
         case OPT_MOTION:                                         /* --motion */
             main_PutPszVariable( MOTION_METHOD_VAR, optarg );
             break;
@@ -642,6 +704,9 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
         case OPT_YUV:                                               /* --yuv */
             main_PutPszVariable( YUV_METHOD_VAR, optarg );
             break;
+        case OPT_SMP:                                               /* --smp */
+            main_PutIntVariable( VDEC_SMP_VAR, atoi(optarg) );
+            break;
 
         /* DVD options */
         case 't':
@@ -650,6 +715,9 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
         case 'T':
             main_PutIntVariable( INPUT_CHAPTER_VAR, atoi(optarg) );
             break;
+        case 'u':
+            main_PutIntVariable( INPUT_ANGLE_VAR, atoi(optarg) );
+            break;
         case 'a':
             if ( ! strcmp(optarg, "ac3") )
                 main_PutIntVariable( INPUT_AUDIO_VAR, REQUESTED_AC3 );
@@ -672,7 +740,7 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
             main_PutPszVariable( INPUT_METHOD_VAR, optarg );
             break;
         case OPT_CHANNELS:                                     /* --channels */
-            p_main->b_channels = 1;
+            main_PutIntVariable( INPUT_NETWORK_CHANNEL_VAR, 1 );
             break;
         case OPT_SERVER:                                         /* --server */
             main_PutPszVariable( INPUT_SERVER_VAR, optarg );
@@ -682,6 +750,7 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
             break;
         case OPT_BROADCAST:                                   /* --broadcast */
             main_PutIntVariable( INPUT_BROADCAST_VAR, 1 );
+            main_PutPszVariable( INPUT_BCAST_ADDR_VAR, optarg );
             break;
 
         /* Synchro options */
@@ -692,9 +761,20 @@ static int GetConfiguration( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] )
         /* Internal error: unknown option */
         case '?':
         default:
+            ShowConsole();
+            RedirectSTDOUT();
             intf_ErrMsg( "intf error: unknown option `%s'",
                          ppsz_argv[optind - 1] );
             Usage( USAGE );
+#ifdef WIN32        /* Pause the console because it's destroyed when we exit */
+            if( strcmp( "", main_GetPszVariable( INTF_STDOUT_VAR,
+                                                 INTF_STDOUT_DEFAULT ) ) == 0 )
+            {
+                /* No stdout redirection has been asked for */
+                intf_MsgImm( "\nPress the RETURN key to continue..." );
+                getchar();
+            }
+#endif
             return( EINVAL );
             break;
         }
@@ -749,10 +829,14 @@ static void Usage( int i_fashion )
     intf_MsgImm( "\nOptions:"
           "\n  -I, --intf <module>            \tinterface method"
           "\n  -v, --verbose                  \tverbose mode (cumulative)"
+          "\n      --stdout <filename>        \tredirect console stdout"
           "\n"
           "\n      --noaudio                  \tdisable audio"
           "\n  -A, --aout <module>            \taudio output method"
           "\n      --stereo, --mono           \tstereo/mono audio"
+          "\n      --spdif                    \tAC3 pass-through mode"
+          "\n      --downmix <module>         \tAC3 downmix method"
+          "\n      --imdct <module>           \tAC3 IMDCT method"
           "\n"
           "\n      --novideo                  \tdisable video"
           "\n  -V, --vout <module>            \tvideo output method"
@@ -761,20 +845,23 @@ static void Usage( int i_fashion )
           "\n  -g, --grayscale                \tgrayscale output"
           "\n      --fullscreen               \tfullscreen output"
           "\n      --overlay                  \taccelerated display"
+          "\n      --xvadaptor <adaptor>      \tXVideo adaptor"
           "\n      --color                    \tcolor output"
           "\n      --motion <module>          \tmotion compensation method"
           "\n      --idct <module>            \tIDCT method"
           "\n      --yuv <module>             \tYUV method"
           "\n      --synchro <type>           \tforce synchro algorithm"
+          "\n      --smp <number of threads>  \tuse several processors"
           "\n"
           "\n  -t, --dvdtitle <num>           \tchoose DVD title"
           "\n  -T, --dvdchapter <num>         \tchoose DVD chapter"
+          "\n  -u, --dvdangle <num>           \tchoose DVD angle"
           "\n  -a, --dvdaudio <type>          \tchoose DVD audio type"
           "\n  -c, --dvdchannel <channel>     \tchoose DVD audio channel"
           "\n  -s, --dvdsubtitle <channel>    \tchoose DVD subtitle channel"
           "\n"
           "\n      --input                    \tinput method"
-          "\n      --channels                    \tenable channels"
+          "\n      --channels                 \tenable channels"
           "\n      --server <host>            \tvideo server address"
           "\n      --port <port>              \tvideo server port"
           "\n      --broadcast                \tlisten to a broadcast"
@@ -788,15 +875,19 @@ static void Usage( int i_fashion )
 
     /* Interface parameters */
     intf_MsgImm( "\nInterface parameters:"
-        "\n  " INTF_METHOD_VAR "=<method name>          \tinterface method"
-        "\n  " INTF_INIT_SCRIPT_VAR "=<filename>               \tinitialization script"
-        "\n  " INTF_CHANNELS_VAR "=<filename>            \tchannels list" );
+        "\n  " INTF_METHOD_VAR "=<method name>        \tinterface method"
+        "\n  " INTF_INIT_SCRIPT_VAR "=<filename>              \tinitialization script"
+        "\n  " INTF_CHANNELS_VAR "=<filename>         \tchannels list"
+        "\n  " INTF_STDOUT_VAR "=<filename>           \tredirect console stdout" );
 
     /* Audio parameters */
     intf_MsgImm( "\nAudio parameters:"
         "\n  " AOUT_METHOD_VAR "=<method name>        \taudio method"
         "\n  " AOUT_DSP_VAR "=<filename>              \tdsp device path"
         "\n  " AOUT_STEREO_VAR "={1|0}                \tstereo or mono output"
+        "\n  " AOUT_SPDIF_VAR "={1|0}                 \tAC3 pass-through mode"
+        "\n  " DOWNMIX_METHOD_VAR "=<method name>     \tAC3 downmix method"
+        "\n  " IMDCT_METHOD_VAR "=<method name>       \tAC3 IMDCT method"
         "\n  " AOUT_RATE_VAR "=<rate>             \toutput rate" );
 
     /* Video parameters */
@@ -809,28 +900,31 @@ static void Usage( int i_fashion )
         "\n  " VOUT_GRAYSCALE_VAR "={1|0}             \tgrayscale or color output"
         "\n  " VOUT_FULLSCREEN_VAR "={1|0}            \tfullscreen"
         "\n  " VOUT_OVERLAY_VAR "={1|0}               \toverlay"
+        "\n  " VOUT_XVADAPTOR_VAR "=<adaptor>         \tXVideo adaptor"
         "\n  " MOTION_METHOD_VAR "=<method name>      \tmotion compensation method"
         "\n  " IDCT_METHOD_VAR "=<method name>        \tIDCT method"
         "\n  " YUV_METHOD_VAR "=<method name>         \tYUV method"
-        "\n  " VPAR_SYNCHRO_VAR "={I|I+|IP|IP+|IPB}   \tsynchro algorithm" );
+        "\n  " VPAR_SYNCHRO_VAR "={I|I+|IP|IP+|IPB}   \tsynchro algorithm"
+        "\n  " VDEC_SMP_VAR "=<number of threads>     \tuse several processors" );
 
     /* DVD parameters */
     intf_MsgImm( "\nDVD parameters:"
         "\n  " INPUT_DVD_DEVICE_VAR "=<device>           \tDVD device"
         "\n  " INPUT_TITLE_VAR "=<title>             \ttitle number"
         "\n  " INPUT_CHAPTER_VAR "=<chapter>         \tchapter number"
+        "\n  " INPUT_ANGLE_VAR "=<angle>             \tangle number"
         "\n  " INPUT_AUDIO_VAR "={ac3|lpcm|mpeg|off} \taudio type"
         "\n  " INPUT_CHANNEL_VAR "=[0-15]            \taudio channel"
         "\n  " INPUT_SUBTITLE_VAR "=[0-31]           \tsubtitle channel" );
 
     /* Input parameters */
     intf_MsgImm( "\nInput parameters:"
-        "\n  " INPUT_SERVER_VAR "=<hostname>          \tvideo server"
-        "\n  " INPUT_PORT_VAR "=<port>            \tvideo server port"
-        "\n  " INPUT_IFACE_VAR "=<interface>          \tnetwork interface"
-        "\n  " INPUT_BROADCAST_VAR "={1|0}            \tbroadcast mode"
-        "\n  " INPUT_CHANNEL_SERVER_VAR "=<hostname>     \tchannel server"
-        "\n  " INPUT_CHANNEL_PORT_VAR "=<port>         \tchannel server port" );
+        "\n  " INPUT_SERVER_VAR "=<hostname>         \tvideo server"
+        "\n  " INPUT_PORT_VAR "=<port>               \tvideo server port"
+        "\n  " INPUT_IFACE_VAR "=<interface>         \tnetwork interface"
+        "\n  " INPUT_BCAST_ADDR_VAR "=<addr>         \tbroadcast mode"
+        "\n  " INPUT_CHANNEL_SERVER_VAR "=<hostname> \tchannel server"
+        "\n  " INPUT_CHANNEL_PORT_VAR "=<port>       \tchannel server port" );
 
 }
 
@@ -858,13 +952,15 @@ static void Version( void )
 static void InitSignalHandler( void )
 {
     /* Termination signals */
-    signal( SIGHUP,  FatalSignalHandler );
+#ifndef WIN32
     signal( SIGINT,  FatalSignalHandler );
+    signal( SIGHUP,  FatalSignalHandler );
     signal( SIGQUIT, FatalSignalHandler );
 
     /* Other signals */
     signal( SIGALRM, SimpleSignalHandler );
     signal( SIGPIPE, SimpleSignalHandler );
+#endif
 }
 
 
@@ -891,9 +987,11 @@ static void FatalSignalHandler( int i_signal )
     /* Once a signal has been trapped, the termination sequence will be
      * armed and following signals will be ignored to avoid sending messages
      * to an interface having been destroyed */
-    signal( SIGHUP,  SIG_IGN );
+#ifndef WIN32
     signal( SIGINT,  SIG_IGN );
+    signal( SIGHUP,  SIG_IGN );
     signal( SIGQUIT, SIG_IGN );
+#endif
 
     /* Acknowledge the signal received */
     intf_ErrMsgImm( "intf error: signal %d received, exiting", i_signal );
@@ -903,6 +1001,34 @@ static void FatalSignalHandler( int i_signal )
     p_main->p_intf->b_die = 1;
 }
 
+/*****************************************************************************
+ * InstructionSignalHandler: system signal handler
+ *****************************************************************************
+ * This function is called when a illegal instruction signal is received by
+ * the program.
+ * We use this function to test OS and CPU_Capabilities
+ *****************************************************************************/
+static void InstructionSignalHandler( int i_signal )
+{
+    /* Once a signal has been trapped, the termination sequence will be
+     * armed and following signals will be ignored to avoid sending messages
+     * to an interface having been destroyed */
+
+    /* Acknowledge the signal received */
+    fprintf( stderr, "warning: extended instructions unsupported, "
+                     "some optimizations will be disabled\n" );
+#ifdef SYS_LINUX
+    fprintf( stderr, "upgrade to kernel 2.4.x to get rid of this warning\n" );
+#endif
+
+    i_illegal = 1;
+    
+#ifdef HAVE_SIGRELSE
+    sigrelse( i_signal );
+#endif
+    longjmp( env, 1 );
+}
+
 /*****************************************************************************
  * CPUCapabilities: list the processors MMX support and other capabilities
  *****************************************************************************
@@ -910,15 +1036,16 @@ static void FatalSignalHandler( int i_signal )
  *****************************************************************************/
 static int CPUCapabilities( void )
 {
-    int i_capabilities = CPU_CAPABILITY_NONE;
+    volatile int i_capabilities = CPU_CAPABILITY_NONE;
 
 #if defined( SYS_BEOS )
     i_capabilities |= CPU_CAPABILITY_486
                       | CPU_CAPABILITY_586
                       | CPU_CAPABILITY_MMX;
 
-#elif defined( SYS_DARWIN1_3 )
+    return( i_capabilities );
 
+#elif defined( SYS_DARWIN )
     struct host_basic_info hi;
     kern_return_t          ret;
     host_name_port_t       host;
@@ -934,7 +1061,7 @@ static int CPUCapabilities( void )
 
     if( ret != KERN_SUCCESS )
     {
-        intf_ErrMsg( "error: couldn't get CPU information" );
+        fprintf( stderr, "error: couldn't get CPU information\n" );
         return( i_capabilities );
     }
 
@@ -946,19 +1073,14 @@ static int CPUCapabilities( void )
         i_capabilities |= CPU_CAPABILITY_ALTIVEC;
     }
 
+    return( i_capabilities );
+
 #elif defined( __i386__ )
-    unsigned int  i_eax, i_ebx, i_ecx, i_edx;
-    boolean_t     b_amd;
-
-#   define cpuid( a )              \
-    asm volatile ( "cpuid"         \
-                 : "=a" ( i_eax ), \
-                   "=b" ( i_ebx ), \
-                   "=c" ( i_ecx ), \
-                   "=d" ( i_edx )  \
-                 : "a"  ( a )      \
-                 : "cc" );         \
+    volatile unsigned int  i_eax, i_ebx, i_ecx, i_edx;
+    volatile boolean_t     b_amd;
 
+    signal( SIGILL, InstructionSignalHandler );
+    
     /* test for a 486 CPU */
     asm volatile ( "pushfl\n\t"
                    "popl %%eax\n\t"
@@ -975,6 +1097,7 @@ static int CPUCapabilities( void )
 
     if( i_eax == i_ebx )
     {
+        signal( SIGILL, NULL );     
         return( i_capabilities );
     }
 
@@ -985,6 +1108,7 @@ static int CPUCapabilities( void )
 
     if( !i_eax )
     {
+        signal( SIGILL, NULL );     
         return( i_capabilities );
     }
 
@@ -1000,6 +1124,7 @@ static int CPUCapabilities( void )
 
     if( ! (i_edx & 0x00800000) )
     {
+        signal( SIGILL, NULL );     
         return( i_capabilities );
     }
 
@@ -1008,6 +1133,19 @@ static int CPUCapabilities( void )
     if( i_edx & 0x02000000 )
     {
         i_capabilities |= CPU_CAPABILITY_MMXEXT;
+
+        /* We test if OS support the SSE instructions */
+        i_illegal = 0;
+        if( setjmp( env ) == 0 )
+        {
+            /* Test a SSE instruction */
+            __asm__ __volatile__ ( "xorps %%xmm0,%%xmm0\n" : : );
+        }
+
+        if( i_illegal == 0 )
+        {
+            i_capabilities |= CPU_CAPABILITY_SSE;
+        }
     }
     
     /* test for additional capabilities */
@@ -1015,6 +1153,7 @@ static int CPUCapabilities( void )
 
     if( i_eax < 0x80000001 )
     {
+        signal( SIGILL, NULL );     
         return( i_capabilities );
     }
 
@@ -1023,7 +1162,17 @@ static int CPUCapabilities( void )
 
     if( i_edx & 0x80000000 )
     {
-        i_capabilities |= CPU_CAPABILITY_3DNOW;
+        i_illegal = 0;
+        if( setjmp( env ) == 0 )
+        {
+            /* Test a 3D Now! instruction */
+            __asm__ __volatile__ ( "pfadd %%mm0,%%mm0\n" "femms\n" : : );
+        }
+
+        if( i_illegal == 0 ) 
+        {
+            i_capabilities |= CPU_CAPABILITY_3DNOW;
+        }
     }
 
     if( b_amd && ( i_edx & 0x00400000 ) )
@@ -1031,10 +1180,74 @@ static int CPUCapabilities( void )
         i_capabilities |= CPU_CAPABILITY_MMXEXT;
     }
 
+    signal( SIGILL, NULL );     
+    return( i_capabilities );
+
 #else
     /* default behaviour */
+    return( i_capabilities );
 
 #endif
-    return( i_capabilities );
+}
+
+/*****************************************************************************
+ * RedirectSTDOUT: redirect stdout and stderr to a file
+ *****************************************************************************
+ * This function will redirect stdout and stderr to a file if the user has
+ * specified so.
+ *****************************************************************************/
+static int RedirectSTDOUT( void )
+{
+    int  i_stdout_filedesc;
+    char *psz_stdout_filename;
+
+    psz_stdout_filename = main_GetPszVariable( INTF_STDOUT_VAR,
+                                               INTF_STDOUT_DEFAULT );
+    if( strcmp( "", psz_stdout_filename ) != 0 )
+    {
+        ShowConsole();
+        i_stdout_filedesc = open( psz_stdout_filename,
+                                  O_CREAT | O_TRUNC | O_RDWR,
+                                  S_IREAD | S_IWRITE );
+
+        if( dup2( i_stdout_filedesc, fileno(stdout) ) == -1 )
+        {
+            intf_ErrMsg( "warning: unable to redirect stdout" );
+        }
+
+        if( dup2( i_stdout_filedesc, fileno(stderr) ) == -1 )
+        {
+            intf_ErrMsg( "warning: unable to redirect stderr" );
+        }
+
+        close( i_stdout_filedesc );
+    }
+    else
+    {
+        /* No stdout redirection has been asked so open a console */
+        if( p_main->i_warning_level )
+        {
+            ShowConsole();
+        }
+
+    }
+
+    return 0;
+}
+
+/*****************************************************************************
+ * ShowConsole: On Win32, create an output console for debug messages
+ *****************************************************************************
+ * This function is usefull only on Win32.
+ *****************************************************************************/
+static void ShowConsole( void )
+{
+#ifdef WIN32 /*  */
+    AllocConsole();
+    freopen( "CONOUT$", "w", stdout );
+    freopen( "CONOUT$", "w", stderr );
+    freopen( "CONIN$", "r", stdin );
+#endif
+    return;
 }
 
